/*The code was written and debugged by John C. Martin at the University of
  Minnesota (copyright 2005).  You are free to distribute this source
  as long as this notice remains in tact.  End users of this code agree to
  take responsibility for how this code is used and John C. Martin and the
  University of Minnesota bear no liability.*/

/*ADDWAVE:  A program to reconstruct and add a WAVE FITS HDU to the end
            of a FITS file.*/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "fitsio.h"

int main(int argc, char *argv[])
{
  char *fln,*cmd;
  fitsfile *fptr;
  int sl,rz,i;
  int naxis,type,hdu=2;
  long naxes[3];
  double crpix1,crval1,cd11,*wave;
  int status=0;
  int debug=0;
 
  fln=(char *)malloc(sizeof(char)*200);
  cmd=(char*)malloc(sizeof(char)*200);

  if (argc<1) {
    printf("usage:  addwave <FITS_filename>\n    Reconstructs WAVE array from WCS information.\n   *IMPORTANT*  This program requires that gunzip be in your path.\n\n");
    return(1);
  }

  strcpy(fln,argv[1]);
  /*Detect if it's a gzip file and unzip it*/
  sl=strlen(fln);
  if (strcmp(fln+(sl-3),".gz")==0) {
    if (debug) printf("Unzipping file....");
    sprintf(cmd,"gunzip %s",fln);
    system(cmd);
    fln[sl-3]=(char)NULL;
    if (debug) printf(" .. done  ");
    rz=1;
  } else rz=0;

  /*Open fits file*/
  if (ffopen(&fptr,fln,READWRITE,&status)) {
    ffrprt(stderr,status);
    fprintf(stderr,"Bad Filename: %s\n",fln);
    return(-1);
  }
  else {
    /*Check to see if a WAVE HDU already exists*/
    ffmnhd(fptr,ANY_HDU,"WAVE",0,&status);
    if (status!=BAD_HDU_NUM) {
      printf("%s WAVE HDU is already exists...aborting\n",fln);
      ffclos(fptr,&status);
      /*rezipping the FITS file*/
      if (rz>0) {
	if (debug) printf("  Zipping...");
	sprintf(cmd,"gzip %s",fln);
	system(cmd);
	if (debug) printf("Done  \n");
      }
      return(-1);
    } 
    status=0;
    ffmahd(fptr,hdu,&type,&status);
    ffgidm(fptr,&naxis,&status);
    ffgisz(fptr,3,naxes,&status);
    ffgky(fptr,TDOUBLE,"CRPIX1",&crpix1,cmd,&status);
    ffgky(fptr,TDOUBLE,"CRVAL1",&crval1,cmd,&status);
    ffgky(fptr,TDOUBLE,"CD1_1",&cd11,cmd,&status);
    naxes[1]=0;
    naxes[2]=0;
    ffcrim(fptr,-64,1,naxes,&status);
    ffpky(fptr,TSTRING,"EXTNAME","WAVE"," ",&status);
    wave=(double*)malloc(sizeof(double)*(naxes[0]+1));
    for (i=0;i<naxes[0];i++) wave[i]=crval1+(i-crpix1+1)*cd11;
    ffppr(fptr,TDOUBLE,1,naxes[0],wave,&status);
    ffphis(fptr,"This WAVE HDU was reconstituted from the FLUX WCS info",&status);
    ffmahd(fptr,2,NULL,&status);
    i=0;
    while(status==0) {
      i++;
      ffmrhd(fptr,1,NULL,&status);
    }
    status=0;
    ffmahd(fptr,1,NULL,&status);
    ffuky(fptr,TINT,"NEXTEND",&i,"Num of stndrd extnsns (excldng prmry HDU)",&status);
    ffphis(fptr,"The WAVE HDU was reconstituted from the FLUX WCS info",&status);
    ffclos(fptr,&status);
    free(wave);
    if (status)  ffrprt(stderr, status); /* print any error message */
  }


  /*rezipping the FITS file*/
  if (rz>0) {
    if (debug) printf("  Zipping...");
    sprintf(cmd,"gzip %s",fln);
    system(cmd);
    if (debug) printf("Done  \n");
  }


  return(0);
}
